package be.iminds.ilabt.jfed.highlevel.tasks;

import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.highlevel.util.AggregateManagerWrapperFactory;
import be.iminds.ilabt.jfed.lowlevel.connection.JFedException;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.AggregateManagerWrapper;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Collections;
import java.util.List;

/**
 * User: twalcari
 * Date: 6/17/2015
 * Time: 16:42
 */
public final class ListSliceResourcesTask extends Task {
    @Nonnull private final Server auth;
    @Nonnull private final Slice slice;

    private String rspec;

    @Nonnull private final AggregateManagerWrapperFactory aggregateManagerWrapperFactory;
    @Nonnull private final HighLevelTaskFactory highLevelTaskFactory;

    private boolean hardFailure;

    public ListSliceResourcesTask(@Nonnull Server auth,
                                  @Nonnull Slice slice,
                                  @Nonnull AggregateManagerWrapperFactory aggregateManagerWrapperFactory,
                                  @Nonnull HighLevelTaskFactory highLevelTaskFactory) {
        super("ListResources of Slice @ " + auth.getDefaultComponentManagerUrn());
        this.auth = auth;
        this.slice = slice;
        this.aggregateManagerWrapperFactory = aggregateManagerWrapperFactory;
        this.highLevelTaskFactory = highLevelTaskFactory;

        this.hardFailure = false;
    }

    @Override
    protected void doTask(@Nonnull TaskExecution taskExecution) throws JFedException, InterruptedException {
        AggregateManagerWrapper amWrapper =
                aggregateManagerWrapperFactory.getAggregateManagerWrapper(taskExecution, auth);

        //make sure that calling before task is ready result in hard failure (= caller called at bad time)
        hardFailure = true;

        try {
            rspec = amWrapper.describe(slice.getUrn(), slice.getCredentials());
            hardFailure = false;
        } catch (JFedException e) {
            hardFailure = true;
        }
    }

    @Nullable public String getRspec() {
        return rspec;
    }

    @Nonnull
    @Override
    public List<Task> initDependsOn() {
        return Collections.singletonList(highLevelTaskFactory.getSliceCredential(slice));
    }

    @Nonnull public Server getAuthority() {
        return auth;
    }

    public boolean isHardFailure() {
        return hardFailure;
    }
}
